package edu.uoc.tfc.sosuna.gestreform.service.impl;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import edu.uoc.tfc.sosuna.gestreform.model.Pressupost;
import edu.uoc.tfc.sosuna.gestreform.persistence.dao.PressupostDao;
import edu.uoc.tfc.sosuna.gestreform.service.PressupostService;
import edu.uoc.tfc.sosuna.gestreform.service.ServiceResults;

/**
 * Implementacio de la API de negoci per a gestionar entitats del tipus Pressupost.
 */
@Service
@Transactional
public class PressupostServiceImpl implements PressupostService{

    private final Logger log = LoggerFactory.getLogger(PressupostServiceImpl.class);
    
    @Inject
    private PressupostDao pressupostDao;
    
    
    /**
     * Crear un pressupost.
     * 
     * @param pressupost L'entitat trobada
     * @return L'entitat creada
     */
    @Override
	public ServiceResults<Pressupost> crea(Pressupost pressupost) {
        log.debug("Creant Pressupost : {}", pressupost);
        pressupost = pressupostDao.save(pressupost);
        return new ServiceResults<Pressupost>(pressupost);
    }

    /**
     * Desar un pressupost.
     * 
     * @param pressupost L'entitat a modificar
     * @return L'entitat modificada
     */
    @Override
	public ServiceResults<Pressupost> modifica(Pressupost pressupost) {
        log.debug("Modificant Pressupost : {}", pressupost);
        pressupost = pressupostDao.save(pressupost);
        return new ServiceResults<Pressupost>(pressupost);
    }

    /**
     *  Obtenir tots els pressupostos.
     *  
     *  @param pageable Dades de paginacio
     *  @return Llista d'entitats trobades
     */
    @Override
	@Transactional(readOnly = true) 
    public ServiceResults<Page<Pressupost>> llista(Pageable pageable) {
        log.debug("Obtenint tots els Pressuposts");
        Page<Pressupost> result = pressupostDao.findAll(pageable); 
        return new ServiceResults<Page<Pressupost>>(result);
    }

    /**
     *  Obtenir tots els pressupostos.
     *  
     *  @param pageable Dades de paginacio
     *  @return Llista d'entitats trobades
     */
    @Override
	@Transactional(readOnly = true) 
    public ServiceResults<Page<Object[]>> llista_variant_1(Pageable pageable) {
        log.debug("Obtenint tots els Pressuposts");
        Page<Pressupost> result = pressupostDao.findAll(pageable); 
        
        List<Object[]> resultList = new ArrayList<>();
        for (Pressupost p : result.getContent()) {
        	resultList.add(new Object[]{
        			p,
        			p.getClient()
        	});
        }
        
        Page<Object[]> result2 = new PageImpl<>(resultList, pageable, result.getTotalElements());
        return new ServiceResults<Page<Object[]>>(result2);
    }
    
    /**
     *  Obtenir un pressupost per el ID.
     *
     *  @param id El ID de l'entitat
     *  @return L'entitat trobada
     */
    @Override
	@Transactional(readOnly = true) 
    public ServiceResults<Pressupost> consulta(Long id) {
        log.debug("Obtenint per ID el Pressupost : {}", id);
        Pressupost pressupost = pressupostDao.findWithClients(id);
        return new ServiceResults<Pressupost>(pressupost);
    }

}
