package edu.uoc.tfc.sosuna.gestreform.rest.config;

import javax.inject.Inject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.security.Http401AuthenticationEntryPoint;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;

import edu.uoc.tfc.sosuna.gestreform.rest.security.OAuth2LogoutHandler;

@Configuration
@EnableResourceServer
public class OAuth2ResourceServerConfiguration extends ResourceServerConfigurerAdapter {

	private Logger log = LoggerFactory.getLogger(OAuth2ResourceServerConfiguration.class);
	
    @Inject
    private OAuth2LogoutHandler oauth2LogoutHandler;

    @Override
    public void configure(HttpSecurity http) throws Exception {
        log.debug("INICI: Configuracio de la seguretat del API REST");
    	
    	http
            // Configuracio dels errors de autenticacio/autoritzacio
        	.exceptionHandling()
            	/*
            	 * Configura SpringSecurity per a retorna error Http 401 (Unauthorized) per a accessos no autenticats encara.
            	 * Per defecte SpringSecurity respon amb Http 403 (Forbidden) a aquests accesos, 
            	 * com si es tractes de URLs internes, prohibides a tothom.
            	 * Com que fem servir SpringSecurity per a controlar l'acces a l'API REST, volem fer servir 
            	 * el Http 401 per a denotar que es un problema d'autoritzacio.
            	 */
        		.authenticationEntryPoint(new Http401AuthenticationEntryPoint(null))	// El null correspon al valor del header "WWW-Authenticate". Aqui no es rellevant perque el codi client no el fa servir.
        .and()
        	// Configuracio del logout
            .logout()
                .logoutUrl("/logout")
                .logoutSuccessHandler(oauth2LogoutHandler)		// Aquest handler s'encarregara d'eliminar el token de OAuth2 de la BD quan s'accedeixi a aquesta URL.
        .and()
        	// La proteccio CSRF esta habilitada per defecte.
        	// Com que no treballem amb cookies i formularis web no la necessitem.
        	//		https://docs.spring.io/spring-security/site/docs/current/reference/html/csrf.html#csrf-configure
            .csrf().disable()
        .sessionManagement()
            // L'estat de l'aplicacio per a cada client que hi accedeix s'emmagatzema al frontend
            // El backend s'encarregara unicament de respondre a peticions al API REST.
            // De manera que mai no farem servir la sessio Java
            // Amb aquesta configuracio ens estalviem els recursos de memoria i proces que suposen les sessions Java, 
            // que normalment es creen automaticament. 
        	.sessionCreationPolicy(SessionCreationPolicy.STATELESS)
        .and()
            .authorizeRequests()
            // Acces public per al endpoint d'autenticacio.
            .antMatchers("/oauth/**").permitAll()
            // Per accedir a la resta del API REST cal haver estat autenticat abans.
            .antMatchers(RestApiConstants.API_ROOT+"/**").authenticated();					
    	log.debug("FI: Configuracio de la seguretat del API REST");
    }
    
}