package edu.uoc.tfc.sosuna.gestreform.rest.service;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import javax.inject.Inject;
import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import edu.uoc.tfc.sosuna.gestreform.model.Client;
import edu.uoc.tfc.sosuna.gestreform.rest.config.RestApiConstants;
import edu.uoc.tfc.sosuna.gestreform.rest.dto.ClientDTO;
import edu.uoc.tfc.sosuna.gestreform.rest.mapper.ClientMapper;
import edu.uoc.tfc.sosuna.gestreform.rest.util.NotificationHeadersUtil;
import edu.uoc.tfc.sosuna.gestreform.service.ClientService;
import edu.uoc.tfc.sosuna.gestreform.service.ServiceResults;

/**
 * REST controller per gestionar entitats del tipus Client.
 */
@RestController
@RequestMapping(RestApiConstants.API_ROOT)
public class ClientRestResource {

    private final Logger log = LoggerFactory.getLogger(ClientRestResource.class);
        
    @Inject
    private ClientService clientService;
    
    @Inject
    private ClientMapper clientMapper;
    
    /**
     * POST  /clients : Crear un nou client.
     */
    @RequestMapping(value = "/clients",
        method = RequestMethod.POST,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ClientDTO> crea(@Valid @RequestBody ClientDTO clientDTO) throws URISyntaxException {
        log.debug("REST  - Crear un nou Client : {}", clientDTO);
        
        clientDTO.setId(null);
        Client client = clientMapper.clientDTOToClient(clientDTO);
        ServiceResults<Client> sr = clientService.crea(client);
        client = sr.getResultData();
        clientDTO = (client!=null? clientMapper.clientToClientDTO(client) : null);

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.created(new URI(RestApiConstants.API_ROOT+"/clients/" + clientDTO.getId()))
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityCreationMessage("client", clientDTO.getId().toString()))
                    .body(clientDTO);
        }
    }

    /**
     * PUT  /clients : Actualitza un client.
     */
    @RequestMapping(value = "/clients",
        method = RequestMethod.PUT,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ClientDTO> modifica(@Valid @RequestBody ClientDTO clientDTO) throws URISyntaxException {
        log.debug("REST- Actualitzar un Client : {}", clientDTO);
        
        if (clientDTO.getId() == null) {
            return crea(clientDTO);
        }
        
        Client client = clientMapper.clientDTOToClient(clientDTO);
        ServiceResults<Client> sr = clientService.modifica(client);
        clientDTO = clientMapper.clientToClientDTO(sr.getResultData());

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityUpdateMessage("client", clientDTO.getId().toString()))
                    .body(clientDTO);
        }
    }

    /**
     * GET  /clients : Llistat de clients.
     */
    @RequestMapping(value = "/clients",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @Transactional(readOnly = true)
    public ResponseEntity<List<ClientDTO>> llista(Pageable pageable)
        throws URISyntaxException {
        log.debug("REST - Llistar els Clients");
        
        ServiceResults<Page<Client>> sr = clientService.llista(pageable); 
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            List<ClientDTO> dtos = clientMapper.clientsToClientDTOs(sr.getResultData().getContent());
        	return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(dtos);
        }
    }

    /**
     * GET  /clients/:id : Obtenir un client.
     */
    @SuppressWarnings("unchecked")
	@RequestMapping(value = "/clients/{id}",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ClientDTO> consulta(@PathVariable Long id) {
        log.debug("REST - Obtenir un Client : {}", id);

        ServiceResults<Client> sr = clientService.consulta(id);
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else if (sr.getResultData()==null) {
        	ResponseEntity<?> r = ResponseEntity
        			.notFound()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .build();
        	return (ResponseEntity<ClientDTO>) r;
        }
        else {
        	ClientDTO clientDTO = clientMapper.clientToClientDTO(sr.getResultData());
        	return ResponseEntity
        			.ok()
        			.headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
        			.body(clientDTO);
        }
    }

}
