package edu.uoc.tfc.sosuna.gestreform.rest.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestController;

import edu.uoc.tfc.sosuna.gestreform.rest.util.NotificationHeadersUtil;

/**
 * AOP per a tractar de forma uniforme els errors a tots els endpoints als Controller
 */
@ControllerAdvice
public class ControllerEndpointExceptionInterceptor {
    
    /**
     * Per tractar errors de validacio JSR-303
     * 
     * @param ex
     * @return
     */
	@ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseEntity<Void> handleValidationError(MethodArgumentNotValidException ex) {
    	// Info de debug
    	LoggerFactory.getLogger(ex.getParameter().getContainingClass()).debug("Peticio rebutjada per fallar la validacio de dades", ex);;
    	
        return ResponseEntity
        		.badRequest()
        		.headers(NotificationHeadersUtil.createError("Error de validacio a les dades", null))
        		.build();
    }

    /**
     * Per tractar errors d'autoritzacio llencats manualment (apart de SpringSecurity)
     * 
     * @param ex
     * @return
     * @throws ClassNotFoundException
     */
	@ExceptionHandler(AccessDeniedException.class)
    public ResponseEntity<Void> handleAccessDeniedException(AccessDeniedException ex) throws ClassNotFoundException {
    	// Info de debug
		getLogger(ex).debug("Peticio rebutjada per manca de privilegis d'acces al endpoint", ex);
    	
        return ResponseEntity
        		.status(HttpStatus.FORBIDDEN)
        		.headers(NotificationHeadersUtil.createError("Error de seguretat en l'acces a l'aplicacio", null))
        		.build();
    }

    /**
     * Tractament general d'errors
     * 
     * @param ex
     * @return
     * @throws ClassNotFoundException
     */
	@ExceptionHandler(Exception.class)
    public ResponseEntity<?> handleUnknownException(Exception ex) throws ClassNotFoundException {
    	// Info de debug
    	getLogger(ex).error("Error inesperat en el processament de la peticio", ex);

        return ResponseEntity
        		.status(HttpStatus.INTERNAL_SERVER_ERROR)
        		.headers(NotificationHeadersUtil.createError("Error intern del servidor", null))
        		.build();
    }
	
	private Logger getLogger(Exception ex) {
		Class<?> c = findLoggerClass(ex);
		return LoggerFactory.getLogger(c);
	}
	
	private Class<?> findLoggerClass(Exception ex) {
		// Busquem una classe anotada amb @Controller
		Class<?> clazz = null;
		for (StackTraceElement t : ex.getStackTrace()) {
			try {
				clazz = Class.forName(t.getClassName());
				if (clazz.isAnnotationPresent(RestController.class)) {
					return clazz;
				}
			}
			catch(ClassNotFoundException e) {
				// Pasapalabra
			} 
		}

		// Si no la trobem, provem obtenir la 1a de la traca
		try {
			return Class.forName(ex.getStackTrace()[0].getClassName());
		}
		catch (ClassNotFoundException e) {
			// Si aixi tampoc, fem log desde aqui
			return this.getClass();
		}
	}
    
}
