package edu.uoc.tfc.sosuna.gestreform.rest.service;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import javax.inject.Inject;
import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import edu.uoc.tfc.sosuna.gestreform.model.Manobre;
import edu.uoc.tfc.sosuna.gestreform.rest.config.RestApiConstants;
import edu.uoc.tfc.sosuna.gestreform.rest.dto.ManobreDTO;
import edu.uoc.tfc.sosuna.gestreform.rest.mapper.ManobreMapper;
import edu.uoc.tfc.sosuna.gestreform.rest.util.NotificationHeadersUtil;
import edu.uoc.tfc.sosuna.gestreform.service.ManobreService;
import edu.uoc.tfc.sosuna.gestreform.service.ServiceResults;

/**
 * REST controller per gestionar entitats del tipus Manobre.
 */
@RestController
@RequestMapping(RestApiConstants.API_ROOT)
public class ManobreRestResource {

    private final Logger log = LoggerFactory.getLogger(ManobreRestResource.class);
        
    @Inject
    private ManobreService manobreService;

    @Inject
    private ManobreMapper manobreMapper;

    
    /**
     * POST  /manobres : Crear un nou manobre.
     */
    @RequestMapping(value = "/manobres",
        method = RequestMethod.POST,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ManobreDTO> crea(@Valid @RequestBody ManobreDTO manobreDTO) throws URISyntaxException {
        log.debug("REST  - Crear un nou Manobre : {}", manobreDTO);

        manobreDTO.setId(null);
        Manobre manobre = manobreMapper.manobreDTOToManobre(manobreDTO);
        ServiceResults<Manobre> sr = manobreService.crea(manobre);
        manobre = sr.getResultData();
        manobreDTO = (manobre!=null? manobreMapper.manobreToManobreDTO(manobre) : null);

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.created(new URI(RestApiConstants.API_ROOT+"/manobres/" + manobreDTO.getId()))
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityCreationMessage("manobre", manobreDTO.getId().toString()))
                    .body(manobreDTO);
        }
    }

    /**
     * PUT  /manobres : Actualitza un manobre.
     */
    @RequestMapping(value = "/manobres",
        method = RequestMethod.PUT,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ManobreDTO> modifica(@Valid @RequestBody ManobreDTO manobreDTO) throws URISyntaxException {
        log.debug("REST- Actualitzar un Manobre : {}", manobreDTO);

        if (manobreDTO.getId() == null) {
            return crea(manobreDTO);
        }
        
        Manobre manobre = manobreMapper.manobreDTOToManobre(manobreDTO);
        ServiceResults<Manobre> sr = manobreService.modifica(manobre);
        manobreDTO = manobreMapper.manobreToManobreDTO(sr.getResultData());

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityUpdateMessage("manobre", manobreDTO.getId().toString()))
                    .body(manobreDTO);
        }
    }

    /**
     * GET  /manobres : Llistat de manobres.
     */
    @RequestMapping(value = "/manobres",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @Transactional(readOnly = true)
    public ResponseEntity<List<ManobreDTO>> llista(Pageable pageable)
        throws URISyntaxException {
        log.debug("REST - Llistar els Manobres");
        
        ServiceResults<Page<Manobre>> sr = manobreService.llista(pageable); 
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            List<ManobreDTO> dtos = manobreMapper.manobresToManobreDTOs(sr.getResultData().getContent());
        	return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(dtos);
        }
    }

    /**
     * GET  /manobres/:id : Obtenir un manobre.
     */
    @SuppressWarnings("unchecked")
	@RequestMapping(value = "/manobres/{id}",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ManobreDTO> consulta(@PathVariable Long id) {
        log.debug("REST - Obtenir un Manobre : {}", id);
        
        ServiceResults<Manobre> sr = manobreService.consulta(id);
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else if (sr.getResultData()==null) {
        	ResponseEntity<?> r = ResponseEntity
        			.notFound()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .build();
        	return (ResponseEntity<ManobreDTO>) r;
        }
        else {
        	ManobreDTO manobreDTO = manobreMapper.manobreToManobreDTO(sr.getResultData());
        	return ResponseEntity
        			.ok()
        			.headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
        			.body(manobreDTO);
        }
    }

}
