package edu.uoc.tfc.sosuna.gestreform.rest.service;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;
import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import edu.uoc.tfc.sosuna.gestreform.model.Client;
import edu.uoc.tfc.sosuna.gestreform.model.Pressupost;
import edu.uoc.tfc.sosuna.gestreform.rest.config.RestApiConstants;
import edu.uoc.tfc.sosuna.gestreform.rest.dto.PressupostDTO;
import edu.uoc.tfc.sosuna.gestreform.rest.mapper.ClientMapper;
import edu.uoc.tfc.sosuna.gestreform.rest.mapper.PressupostMapper;
import edu.uoc.tfc.sosuna.gestreform.rest.util.NotificationHeadersUtil;
import edu.uoc.tfc.sosuna.gestreform.service.PressupostService;
import edu.uoc.tfc.sosuna.gestreform.service.ServiceResults;

/**
 * REST controller per gestionar entitats del tipus Pressupost.
 */
@RestController
@RequestMapping(RestApiConstants.API_ROOT)
public class PressupostRestResource {

    private final Logger log = LoggerFactory.getLogger(PressupostRestResource.class);
        
    @Inject
    private PressupostService pressupostService;
    
    @Inject
    private PressupostMapper pressupostMapper;
    @Inject
    private ClientMapper clientMapper;

    
    /**
     * POST  /pressuposts : Crear un nou pressupost.
     */
    @RequestMapping(value = "/pressuposts",
        method = RequestMethod.POST,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PressupostDTO> crea(@Valid @RequestBody PressupostDTO pressupostDTO) throws URISyntaxException {
        log.debug("REST  - Crear un nou Pressupost : {}", pressupostDTO);
        
        pressupostDTO.setId(null);
        Pressupost pressupost = pressupostMapper.pressupostDTOToPressupost(pressupostDTO);
        ServiceResults<Pressupost> sr = pressupostService.crea(pressupost);
        pressupost = sr.getResultData();
        pressupostDTO = (pressupost!=null? pressupostMapper.pressupostToPressupostDTO(pressupost) : null);

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.created(new URI(RestApiConstants.API_ROOT+"/pressuposts/" + pressupostDTO.getId()))
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityCreationMessage("pressupost", pressupostDTO.getId().toString()))
                    .body(pressupostDTO);
        }
    }

    /**
     * PUT  /pressuposts : Actualitza un pressupost.
     */
    @RequestMapping(value = "/pressuposts",
        method = RequestMethod.PUT,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PressupostDTO> modifica(@Valid @RequestBody PressupostDTO pressupostDTO) throws URISyntaxException {
        log.debug("REST- Actualitzar un Pressupost : {}", pressupostDTO);

        if (pressupostDTO.getId() == null) {
            return crea(pressupostDTO);
        }
        
        Pressupost pressupost = pressupostMapper.pressupostDTOToPressupost(pressupostDTO);
        ServiceResults<Pressupost> sr = pressupostService.modifica(pressupost);
        pressupostDTO = pressupostMapper.pressupostToPressupostDTO(sr.getResultData());

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityUpdateMessage("pressupost", pressupostDTO.getId().toString()))
                    .body(pressupostDTO);
        }
    }

    /**
     * GET  /pressuposts : Llistat de pressupostos
     */
    @RequestMapping(value = "/pressuposts",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @Transactional(readOnly = true)
    public ResponseEntity<?> llista(Pageable pageable, @RequestParam("flavour") String flavour)
        throws URISyntaxException {
        // Multiplexat de diferents cerques mitjancant parametre "flavour"
    	if ("1".equals(flavour)) {
        	return llista_variant_1(pageable);
        }
        else {
        	return llista_normal(pageable);
        }
    }

    /**
     * GET  /pressuposts : Llistat de pressupostos (normal).
     */
    private ResponseEntity<List<PressupostDTO>> llista_normal(Pageable pageable)
        throws URISyntaxException {
        log.debug("REST - Llistar els Pressuposts, format normal");
        
        ServiceResults<Page<Pressupost>> sr = pressupostService.llista(pageable); 
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            List<PressupostDTO> dtos = pressupostMapper.pressupostsToPressupostDTOs(sr.getResultData().getContent());
        	return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(dtos);
        }
    }
    
    
    /**
     * GET  /pressuposts : Llistat de pressupostos (variant "1").
     */
    public ResponseEntity<List<Object[]>> llista_variant_1(Pageable pageable) 
        throws URISyntaxException {
        log.debug("REST - Llistar els Pressuposts (variant '1')");
        
        ServiceResults<Page<Object[]>> sr = pressupostService.llista_variant_1(pageable);
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            List<Object[]> list = new ArrayList<>();
            
            for (Object[] item : sr.getResultData().getContent()) {
            	item[0] = pressupostMapper.pressupostToPressupostDTO((Pressupost) item[0]);
            	item[1] = clientMapper.clientToClientDTO((Client) item[1]);
            	list.add(item);
            }
            
            return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(list);
        }
    }
    
    /**
     * GET  /pressuposts/:id : Obtenir un pressupost.
     */
    @SuppressWarnings("unchecked")
	@RequestMapping(value = "/pressuposts/{id}",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PressupostDTO> consulta(@PathVariable Long id) {
        log.debug("REST - Obtenir un Pressupost : {}", id);
        
        ServiceResults<Pressupost> sr = pressupostService.consulta(id);
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else if (sr.getResultData()==null) {
        	ResponseEntity<?> r = ResponseEntity
        			.notFound()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .build();
        	return (ResponseEntity<PressupostDTO>) r;
        }
        else {
        	PressupostDTO pressupostDTO = pressupostMapper.pressupostToPressupostDTO(sr.getResultData());
        	return ResponseEntity
        			.ok()
        			.headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
        			.body(pressupostDTO);
        }
    }

}
