package edu.uoc.tfc.sosuna.gestreform.rest.service;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import javax.inject.Inject;
import javax.validation.Valid;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import edu.uoc.tfc.sosuna.gestreform.model.Usuari;
import edu.uoc.tfc.sosuna.gestreform.rest.config.RestApiConstants;
import edu.uoc.tfc.sosuna.gestreform.rest.dto.UsuariDTO;
import edu.uoc.tfc.sosuna.gestreform.rest.mapper.UsuariMapper;
import edu.uoc.tfc.sosuna.gestreform.rest.util.NotificationHeadersUtil;
import edu.uoc.tfc.sosuna.gestreform.service.ServiceResults;
import edu.uoc.tfc.sosuna.gestreform.service.UsuariService;

/**
 * REST controller per gestionar entitats del tipus Usuari.
 */
@RestController
@RequestMapping(RestApiConstants.API_ROOT)
public class UsuariRestResource {

    private final Logger log = LoggerFactory.getLogger(UsuariRestResource.class);
        
    @Inject
    private UsuariService usuariService;
    
    @Inject
    private UsuariMapper usuariMapper;
    
    
    /**
     * POST  /usuaris : Crear un nou usuari.
     */
    @RequestMapping(value = "/usuaris",
        method = RequestMethod.POST,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<UsuariDTO> crea(@Valid @RequestBody UsuariDTO usuariDTO) throws URISyntaxException {
        log.debug("REST  - Crear un nou Usuari : {}", usuariDTO);
        
        usuariDTO.setId(null);
        Usuari usuari = usuariMapper.usuariDTOToUsuari(usuariDTO);
        ServiceResults<Usuari> sr = usuariService.crea(usuari);
        usuari = sr.getResultData();
        usuariDTO = (usuari!=null? usuariMapper.usuariToUsuariDTO(usuari) : null);

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.created(new URI(RestApiConstants.API_ROOT+"/usuaris/" + usuariDTO.getId()))
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityCreationMessage("usuari", usuariDTO.getId().toString()))
                    .body(usuariDTO);
        }
    }

    /**
     * PUT  /usuaris : Actualitza un usuari.
     */
    @RequestMapping(value = "/usuaris",
        method = RequestMethod.PUT,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<UsuariDTO> modifica(@Valid @RequestBody UsuariDTO usuariDTO) throws URISyntaxException {
        log.debug("REST- Actualitzar un Usuari : {}", usuariDTO);

        if (usuariDTO.getId() == null) {
            return crea(usuariDTO);
        }
        
        Usuari usuari = usuariMapper.usuariDTOToUsuari(usuariDTO);
        ServiceResults<Usuari> sr = usuariService.modifica(usuari);
        usuariDTO = usuariMapper.usuariToUsuariDTO(sr.getResultData());

        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .headers(NotificationHeadersUtil.createEntityUpdateMessage("usuari", usuariDTO.getId().toString()))
                    .body(usuariDTO);
        }
    }

    /**
     * GET  /usuaris : Llistat de usuaris.
     */
    @RequestMapping(value = "/usuaris",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    @Transactional(readOnly = true)
    public ResponseEntity<List<UsuariDTO>> llista(Pageable pageable, @RequestParam(value="flavour", required=false) String flavour, @RequestParam MultiValueMap<String, String> params)
        throws URISyntaxException {
    	if ("filterByProfile".equals(flavour)) {
    		String profileId = params.getFirst("profile");
    		return llistaPerPerfil(pageable, profileId);
    	}
    	else {
    		return llistaNormal(pageable);
    	}
    }

    private ResponseEntity<List<UsuariDTO>> llistaNormal(Pageable pageable)
        throws URISyntaxException {
        log.debug("REST - Llistar tots els Usuaris");

        ServiceResults<Page<Usuari>> sr = usuariService.llista(pageable); 
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            List<UsuariDTO> dtos = usuariMapper.usuarisToUsuariDTOs(sr.getResultData().getContent());
        	return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(dtos);
        }    
    }
    
    private ResponseEntity<List<UsuariDTO>> llistaPerPerfil(Pageable pageable, String profileId)
        throws URISyntaxException {
        log.debug("REST - Llistar els Usuaris amb el perfil [{}]", profileId);

        ServiceResults<Page<Usuari>> sr = usuariService.llistaPerPerfil(profileId,pageable); 
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else {
            List<UsuariDTO> dtos = usuariMapper.usuarisToUsuariDTOs(sr.getResultData().getContent());
        	return ResponseEntity
            		.ok()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(dtos);
        }    
    }

    
    /**
     * GET  /usuaris/:id : Obtenir un usuari.
     */
    @SuppressWarnings("unchecked")
	@RequestMapping(value = "/usuaris/{id}",
        method = RequestMethod.GET,
        produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<UsuariDTO> consulta(@PathVariable Long id) {
        log.debug("REST - Obtenir un Usuari : {}", id);

        ServiceResults<Usuari> sr = usuariService.consulta(id);
        
        if (sr.hasErrors()) {
            return ResponseEntity
            		.unprocessableEntity()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .body(null);
        }
        else if (sr.getResultData()==null) {
        	ResponseEntity<?> r = ResponseEntity
        			.notFound()
                    .headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
                    .build();
        	return (ResponseEntity<UsuariDTO>) r;
        }
        else {
        	UsuariDTO usuariDTO = usuariMapper.usuariToUsuariDTO(sr.getResultData());
        	return ResponseEntity
        			.ok()
        			.headers(NotificationHeadersUtil.createServiceMessageHeaders(sr))
        			.body(usuariDTO);
        }
    }

}
